<?php

namespace App\Http\Controllers\Client;

use App\Http\Controllers\Controller;
use App\Http\Requests\Client\LoginRequest;
use App\Http\Requests\Client\RegisterRequest;
use App\Http\Resources\ShowProfileResource;
use App\Models\Admin\Structure\SiteInformation;
use App\Models\Company;
use App\Models\Project;
use App\Models\SocialMediable;
use App\Models\User;
use App\Repositories\ReferralCodeRepository;
use App\Repositories\Role\AuthenticationRepository;
use App\Repositories\Role\SearchRepository;
use App\Services\CodeService;
use App\Traits\ApiResponser;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Mojtaba\Chatable\Models\Group;
use Mojtaba\Chatable\Models\Member;
use Symfony\Component\HttpFoundation\Response;

class AuthenticationController extends Controller
{
    use ApiResponser;
    public $request;
    private $repository;
    private $type;
    private $guardName;
    private $className;

    /**
     * @var string
     */

    public function __construct(Request $request, AuthenticationRepository $repository)
    {
        $this->middleware('client')->only('logout');
        $this->request = $request;
        $this->repository = $repository;
    }

    public function register(RegisterRequest $registerRequest, CodeService $codeService, SearchRepository $searchUserRepository, ReferralCodeRepository $referralCodeRepository, $clientRole)
    {
//        Log::info($registerRequest);
        $this->guardName = Str::plural($clientRole);
        $this->className = "\\App\\Models\\" . Str::ucfirst($clientRole);
        $clientRoles = [User::class, Company::class];

//        if ($this->className=='\\'.User::class)
//            if ($this->className::where('name',$this->request['name'])->first())
//                return $this->onFaile('The name must be unique');
        if ($this->className=='\\'.Company::class)
            if ($this->request['name'])
            if ($this->className::where('name',$this->request['name'])->first())
                return $this->onFaile('The name must be unique');

        if (User::where('email',$registerRequest->email)->first()){
            return failedResponse('You are already registered with the user account', Response::HTTP_OK);
        }
        if (Company::where('email',$registerRequest->email)->first()){
            return failedResponse('You are already registered with the company account', Response::HTTP_OK);
        }
//return User::class;
        try {
            DB::beginTransaction();
            //check confirmation code
//           $cacheCode = Cache::get($this->request['email']);
//            if ($cacheCode == null || $cacheCode != $this->request['confirmation_code']) {
//                return failedResponse('you cant verify your email', Response::HTTP_FORBIDDEN);
//            }


            //set referral code
            do {
                $referralCode = $codeService->generateCode();
            } while ($codeService->check($clientRoles, 'referral_code', $referralCode));

            $familiarMethod = null;
            if ($registerRequest->familiar_method )
                foreach ($registerRequest->familiar_method as $item){
                    $setting=   SiteInformation::where('key',$item)->first();
                    if ($setting)
                        $setting->update(['value'=>(int)$setting->value+1]);
                }
            //insert data
            if ($this->className=='\\'.User::class)
            $value = $this->repository->register($this->request, $referralCode, $this->className, $familiarMethod);
            if ($this->className=='\\'.Company::class)
                $value = $this->repository->registerComany($this->request, $referralCode, $this->className, $familiarMethod);
            //check referral code exists
            if ($this->request['referral_code']) {
                $checkCode = $codeService->check($clientRoles, 'referral_code', $this->request['referral_code']);
                if ($checkCode == null)
                    return successResponse(false, null, 'your referral code not belongs to any one');
                $client = $this->className::find($value->id);
                $invitation = get_class($checkCode)::find($checkCode->id);
                $addInvitaion = $codeService->invitation($client, $invitation);
                $countInvitation = $codeService->increaseInvitation($invitation);
            }
            $token = $value->createToken('Api Token', ["$this->guardName"])->plainTextToken;
            DB::commit();
            return successResponse(true,['token' => $token,'data'=>$value->load('address','jobTitle')], 'you registered successfully', Response::HTTP_CREATED);
        } catch (\Exception $e) {
            DB::rollback();
            return failedResponse($e->getMessage(), Response::HTTP_BAD_REQUEST);
        }
    }

    public function login(LoginRequest $loginRequest, SearchRepository $searchRepository, $clientRole)
    {
//    $id=  auth($clientRole)->user();
//        $idClinte = $id->id;
//        return 1;
        $type = "App\\Models\\" . Str::ucfirst($clientRole);

        $this->className = "\\App\\Models\\" . Str::ucfirst($clientRole);
        $this->guardName = Str::plural($clientRole);
        $value = $searchRepository->searchEmail($this->className, $this->request['email']);
        if (!$value || !Hash::check($this->request['password'], $value->password)) {
            return successResponse(false, null, 'your email or password wrong!');
        }
//        return $this->className;
        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$type)->where('social_mediable_id',$value->id)->get();
//return $social;


//        return $value;
        $projects=$value->projects()->count();
        $reports=$value->reports()->count();
        $groups=Member::query()->where('member_id',$value->id)->where('member_type',get_class($value))->count();
        $value['social']=ShowProfileResource::collection($social);
        $value['projects']=$projects;
        $value['reports']=$reports;
        $value['groups']=$groups;


        return successResponse(true, ['token' => $value->createToken('API Token', [$this->guardName])
            ->plainTextToken,
            'data' => $value->load([ 'jobTitle','address'])], 'login');
    }

    public function logout($clientRole)
    {
        $revoke = auth($clientRole)->user()->tokens()->delete();
        return successResponse(true, $revoke, 'Token revoked successfully');
    }
    public function checkOtp(Request $request){
        $cacheCode = Cache::get($request->email);
        if ($cacheCode == null || $cacheCode != $request->confirmation_code) {
            return $this->failedResponse('Wrong code entered', Response::HTTP_OK);
        }
        Cache::forget($request->email);
        return $this->successResponseWithoutMessage(true,null);
    }
}
